package fr.univlyon1.tiw1.inscriptions;

import fr.univlyon1.tiw1.inscriptions.modele.Event;
import fr.univlyon1.tiw1.inscriptions.modele.Person;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import java.util.ArrayList;
import java.util.Collection;

/**
 * Created by ecoquery on 19/10/2016.
 */

public class InscriptionsDAO {

    private EntityManager em;

    public EntityManager getEm() {
        return em;
    }

    @PersistenceContext
    public void setEm(EntityManager em) {
        this.em = em;
    }

    public Collection<Event> getAllEvents() {
        return em.createQuery("SELECT e FROM Event e", Event.class).getResultList();
    }

    public Person getPerson(String email) {
        return em.find(Person.class, email);
    }

    public Event getEvent(int id) {
        return em.find(Event.class, id);
    }

    @Transactional
    public Person createOrUpdate(Person p) {
        Person orig = getPerson(p.getEmail());
        if (orig == null) {
            em.persist(p);
            return p;
        } else {
            // ne pas supprimer les inscriptions
            if (p.getCompany() == null) {
                p.setCompany(orig.getCompany());
            }
            if (p.getName() == null) {
                p.setName(orig.getName());
            }
            Collection<Event> evts = new ArrayList<>(orig.getInscriptions());
            for (Event evt : evts) {
                if (!p.getInscriptions().contains(evt)) {
                    p.inscrire(evt);
                }
            }
            return em.merge(p);
        }
    }

    public Event getEventByName(String title) throws EvenementInconnuException {
        TypedQuery<Event> q = em.createQuery("SELECT e FROM Event e WHERE e.title = ?1", Event.class);
        q.setParameter(1, title);
        q.setMaxResults(1);
        try {
            return q.getSingleResult();
        } catch (NoResultException e) {
            throw new EvenementInconnuException("Pas d'événement intitulé '" + title + "'");
        }

    }
}
